# Cleans up poll data for the 2012 election
# Removes unnecessary columns, formats numbers, sorts by state
# and changes the date representation

# Output schema:
# state_abbrev,dem_percent,rep_percent,start_date,end_date,pollster

# Usage:
# python clean-2012-polls.py [input path] [state abbrev. path]
# input path should contain raw poll data
# state abbrev. path should map from state abbreviations to full names

# Cleaned data is sent to stdout

import sys
import locale
import datetime as dt
from functools import cmp_to_key

DATE_FORMAT = "%b %d"

# Reads state abbreviations from the given path
# Returns a dictionary mapping full name to abbreviation
def get_state_abbrevs(path):
    result = {}
    for line in open(path, "r"):
        parts = line.split(",")
        result[parts[1]] = parts[0]
    return result

# Reads 2012 poll data from data_path, and cleans it.
# abbrev_path is path to state abbreviation file
# Data is returned as a list of tuples in arbitrary order
def clean_2012_polls(data_path, abbrev_path):
    abbrevs = get_state_abbrevs(abbrev_path)

    # We need to special-case DC's full name:
    abbrevs["D.C."] = "DC"

    result = []
    first_line = True
    for line in open(data_path, "r"):
        if first_line == True:
            first_line = False
            continue
        parts = line.split(",")
        state = abbrevs[parts[2]]
        length = int(parts[1])
        dem_percent = parts[4]
        rep_percent = parts[5]

        # Remove the length from the end of the pollster name
        pollster = parts[15].split("-")[0]

        # Parse the end date, and calculate the start date
        end_date = dt.datetime.strptime(parts[7] + " 2012", DATE_FORMAT + " %Y")
        start_date = end_date - dt.timedelta(days=length)
        end_str = end_date.strftime(DATE_FORMAT)
        start_str = start_date.strftime(DATE_FORMAT)

        result.append((state, dem_percent, rep_percent,
                       start_str, end_str, pollster))
    return result

# Main Function
if __name__ == "__main__":
    data_path = sys.argv[1]
    abbrev_path = sys.argv[2]

    data = clean_2012_polls(data_path, abbrev_path)
    data.sort(key=cmp_to_key(lambda t1, t2: locale.strcoll(t1[0], t2[0])))
    for line in data:
        print ",".join(line)
